@@ -3,11 +3,6 @@ author: Mislav Sever
 
 TitaniumCloud
 A Python module for the ReversingLabs TitaniumCloud REST API-s.
-
-Copyright (c) ReversingLabs International GmbH. 2016-2021
-
-This unpublished material is proprietary to ReversingLabs International GmbH.. All rights reserved.
-Reproduction or distribution, in whole or in part, is forbidden except by express written permission of ReversingLabs International GmbH.
 """
 
 import datetime
@@ -53,11 +48,7 @@ class TiCloudAPI(object):
     def __init__(self, host, username, password, verify=True, proxies=None,
                  user_agent=DEFAULT_USER_AGENT, allow_none_return=False):
 
-        if host.startswith("http://"):
-            raise WrongInputError("Unsupported protocol definition: "
-                                  "TitaniumCloud services can only be used over HTTPS.")
-        self._host = self.__format_url(host)
-
+        self._host = self.__validate_host(host)
         self._username = username
         self._password = password
         self._credentials = (self._username, self._password)
@@ -76,16 +67,25 @@ class TiCloudAPI(object):
         self._allow_none_return = allow_none_return
 
     @staticmethod
-    def __format_url(host):
+    def __validate_host(host):
         """Returns a formatted host URL including the protocol prefix.
             :param host: URL string
             :type host: str
             :returns: formatted URL string
             :rtype: str
         """
+        if not isinstance(host, str):
+            raise WrongInputError("host parameter must be string.")
+
+        if host.startswith("http://"):
+            raise WrongInputError("Unsupported protocol definition: "
+                                  "TitaniumCloud services can only be used over HTTPS.")
+
         if not host.startswith("https://"):
             host = "https://{host}".format(host=host)
 
+        host = host.rstrip("/")
+
         return host
 
     def _get_request(self, url):
@@ -1741,14 +1741,14 @@ class DynamicAnalysis(TiCloudAPI):
     def get_dynamic_analysis_results(self, sample_hash, latest=False, analysis_id=None):
         """Returns dynamic analysis results for a desired sample.
         The analysis of the selected sample must be finished for the results to be available.
-        :param sample_hash: SHA-1 hash of a desired sample
-        :type sample_hash: str
-        :param latest: return only the latest analysis results
-        :type latest: bool
-        :param analysis_id: return only the results of this analysis
-        :type analysis_id: str
-        :return: response
-        :rtype: requests.Response
+            :param sample_hash: SHA-1 hash of a desired sample
+            :type sample_hash: str
+            :param latest: return only the latest analysis results
+            :type latest: bool
+            :param analysis_id: return only the results of this analysis
+            :type analysis_id: str
+            :return: response
+            :rtype: requests.Response
         """
         validate_hashes(
             hash_input=[sample_hash],
@@ -1844,6 +1844,76 @@ class CertificateAnalytics(TiCloudAPI):
         return response
 
 
+class RansomwareIndicators(TiCloudAPI):
+    """Ransomware Indicators Feed"""
+
+    __FEED_ENDPOINT = "/api/public/v1/ransomware/indicators?withHealth={health}&tagFormat={tag_format}&" \
+                      "hours={hours_back}&indicatorTypes={indicator_types}&onlyFreemium={only_freemium}"
+
+    def __init__(self, host, username, password, verify=True, proxies=None, user_agent=DEFAULT_USER_AGENT,
+                 allow_none_return=False):
+        super(RansomwareIndicators, self).__init__(host, username, password, verify, proxies, user_agent=user_agent,
+                                                   allow_none_return=allow_none_return)
+
+        self._url = "{host}{{endpoint}}".format(host=self._host)
+        self._allowed_indicator_types = ("ipv4", "hash", "domain", "uri")
+
+    def get_indicators(self, hours_back, indicator_types, tag_format="dict", health_check=0, only_freemium=0):
+        """Accepts a list of indicator type strings and an integer for historical hours.
+        Returns indicators of ransomware and related tools.
+            :param hours_back: historical hours - from this moment back
+            :type hours_back: int
+            :param indicator_types: a list of indicator types to fetch; possible values are 'ipv4', 'hash', 'domain', 'uri'
+            :type indicator_types: list[str]
+            :param tag_format: response format; default is 'dict'
+            :type tag_format: str
+            :param health_check: defines whether this request is an API health check;
+            possible values are 0 and 1; default is 0
+            :type health_check: int
+            :param only_freemium: return only freemium indicators of all types;
+            possible values are 0 and 1: default is 0; if set to 1, parameter indicator_types is ignored
+            :type only_freemium: int
+            :return: response
+            :rtype: requests.Response
+        """
+        if not (isinstance(hours_back, int) and 1 <= hours_back <= 48):
+            raise WrongInputError("hours_back parameter must be integer with a value between 1 and 48.")
+
+        try:
+            lowered_indicator_types = [indicator_type.lower() for indicator_type in indicator_types]
+
+        except (AttributeError, TypeError):
+            raise WrongInputError("indicator_types parameter must be a list of strings.")
+
+        if not all(element in self._allowed_indicator_types for element in lowered_indicator_types):
+            raise WrongInputError("Only the following values are allowed as indicator types: "
+                                  "{indicator_types}".format(indicator_types=self._allowed_indicator_types))
+
+        indicator_types = ",".join(lowered_indicator_types)
+
+        if health_check not in (0, 1):
+            raise WrongInputError("health_check parameter must be 0 or 1.")
+
+        if only_freemium not in (0, 1):
+            raise WrongInputError("only_freemium parameter must be 0 or 1.")
+
+        endpoint = self.__FEED_ENDPOINT.format(
+            health=health_check,
+            tag_format=tag_format,
+            hours_back=hours_back,
+            indicator_types=indicator_types,
+            only_freemium=only_freemium
+        )
+
+        url = self._url.format(endpoint=endpoint)
+
+        response = self._get_request(url=url)
+
+        self._raise_on_error(response)
+
+        return response
+
+
 def _update_hash_object(input_source, hash_object):
     """Accepts a string or an opened file in 'rb' mode and a created hashlib hash object and
     returns an updated hashlib hash object.
